// constants
// c0  - modelviewprojection
// c10 - modelview
// c4  - diffuse color
// c6  - specular color
// c5  - light[0].pos
// c7  - light[1].pos
// c9  - misc constants
// --- defined in the shader
// c14 - white color
// c15 - red color
// c16 - <60, 70, 80, 90>

// register allocation (DON'T CLOBBER)
// r0  - normal
// r1  - vertex position in view space
// r2  - light[0] direction
// r3  - light[1] direction
// r4  - light[0] half
// r5  - light[1] half
// r6  - diffuse color accumulator
// r7  - specular color accumulator
// r8  - view direction
      
vs.1.1						// version instruction

dcl_position v0             // declare register data
dcl_normal v4               // v0 is position, v4 is normal
dcl_texcoord0 v8            // v8 is texture coordinates

def c15, 0, 0, 0, 0

m4x4 oPos, v0, c0			// transform vertices by view/projection matrix
m3x3 r0, v4, c0				// transform the normal by world matrix
m4x4 r1, v0, c10			// transform the vertex by the modelview matrix

// -- normalize normal
dp3 r0.w, r0, r0
rsq r0.w, r0.w
mul r0, r0, r0.w

// -- calculate light direction (light 0)
sub r2, c5, r1
dp3 r2.w, r2, r2
rsq r2.w, r2.w
mul r2, r2, r2.w

// -- calculate illumination
mov r10, c15
dp3 r10.x, r0, r2			// N.L
lit r10, r10				// .y = max(N.L, 0)
mul r6, r10.y, c4			// multiply diffuse

mov oD0, r6