// constants
// c0  - modelviewprojection
// c10 - modelview
// c4  - diffuse color
// c6  - specular color
// c5  - light[0].pos
// c7  - light[1].pos
// c9  - misc constants

// register allocation (DON'T CLOBBER)
// r0  - normal
// r1  - vertex position in view space
// r2  - light[0] direction
// r3  - light[1] direction
// r4  - light[0] half
// r5  - light[1] half
// r6  - diffuse color accumulator
// r7  - specular color accumulator
// r8  - view direction
      
vs.1.1						// version instruction

dcl_position v0             // declare register data
dcl_normal v4               // v0 is position, v4 is normal
dcl_texcoord0 v8            // v8 is texture coordinates

def c16, 1, -1, 0.5, 1
//def c20, 1, 1.2, 1.3333, 1.4
def c20, 5, 1.2, 1.1, 1.2
def c21, 0, 0, -1, 0
def c22, 2, 2, 2, 2
def c25, 0, 0, 0, 0
def c26, 0.25, 0.3, 0.45, 1

m4x4 oPos, v0, c0			// transform vertices by view/projection matrix
m3x3 r0, v4, c0				// transform the normal by world matrix
m4x4 r1, v0, c10			// transform the vertex by the modelview matrix

// -- normalize normal
dp3 r0.w, r0, r0
rsq r0.w, r0.w
mul r0, r0, r0.w

// -- calculate light direction (light 0)
sub r2, c5, r1
dp3 r2.w, r2, r2
rsq r2.w, r2.w
mul r2, r2, r2.w

// -- normalize view direction
dp3 r1.w, r1, r1
rsq r1.w, r1.w
mul r1, r1, r1.w

// -- calculate illumination
mov r10, c25
dp3 r10.x, r0, r2			// N.L
lit r10, r10				// .y = max(N.L, 0)
mul r6, r10.y, c4			// multiply diffuse

mul oD0, r6, c26

// that's the reflected view vector
dp3 r10, r0, c21		// dot(N, V)
mul r10, r0, r10		// * norm
mul r10, r10, c22.x		// * 2
add r10, r10, -c21		// - view
//mov r10, -r10

m3x3 r11, r10, c42		// transform with the inverse view matrix

//mov oT0, r11.xyz
mov oT1, r11.xyz

// -- calculate refr
// r7.x: cos_inc
// r7.y: ior_sq
// r7.z: radical
// r7.w: beta
// r8: ior
// c16: <1, -1>

//rcp r8.x, c20.z		// 1 / ior
mov r8.x, c20.x
mov r1, c21

dp3 r7.x, r1, r0

mul r7.y, r8.x, r8.x
sub r10.x, r7.x, c16.y
mad r7.z, r7.y, r10.x, c16.x

mul r10.x, r8.x, r7.x
rsq r10.y, r7.z
rcp r10.y, r10.y
sub r7.w, r10.x, r10.y

mul r10, r1, r8.x
mad r10, r0, r7.w, r10	// r8: refraction vector

m3x3 r8, -r10, c42		// transform with the inverse view matrix

// calculate fake fresnel term
dp3 r5, r1, r0
sub r5, c16.x, r5
mul r5, r5, r5
mov oD1, r5

//dp3 r5, r1, r0
//mul r5, r5, r5
//mul r5, r5, r5

//add oD0, r5, c20.z
mov oT0, r8.xyz