// constants
// c0  - modelviewprojection
// c10 - modelview
// c4  - diffuse color
// c6  - specular color
// c5  - light[0].pos
// c7  - light[1].pos
// c9  - misc constants
// --- defined in the shader
// c14 - white color
// c15 - red color
// c16 - <60, 70, 80, 90>

// register allocation (DON'T CLOBBER)
// r0  - normal
// r1  - vertex position in view space
// r2  - light[0] direction
// r3  - light[1] direction
// r4  - light[0] half
// r5  - light[1] half
// r6  - diffuse color accumulator
// r7  - specular color accumulator
// r8  - view direction
      
vs.1.1						// version instruction

dcl_position v0             // declare register data
dcl_normal v4               // v0 is position, v4 is normal
dcl_texcoord0 v8            // v8 is texture coordinates

def c13, 0, 0, -1, 0
def c14, 1, 1, 1, 1
def c15, 1, 0, 0, 1
def c16, 60, 70, 80, 90
def c17, 0, 0, 0, 1
def c18, 0.5, 0.5, 0.5, 1

m4x4 oPos, v0, c0			// transform vertices by view/projection matrix
m3x3 r0, v4, c0				// transform the normal by world matrix
m4x4 r1, v0, c10			// transform the vertex by the modelview matrix

// -- normalize normal
dp3 r0.w, r0, r0
rsq r0.w, r0.w
mul r0, r0, r0.w

// -- normalize view direction
mov r8, -r1
dp3 r8.w, r8, r8
rsq r8.w, r8.w
mul r8, r8, r8.w

// -- calculate light direction (light 0)
sub r2, c5, r1
dp3 r2.w, r2, r2
rsq r2.w, r2.w
mul r2, r2, r2.w

// -- calculate half angle vector (light 0)
add r4, r2, r8
dp3 r4.w, r4, r4
rsq r4.w, r4.w
mul r4, r4, r4.w

// -- calculate illumination
mov r10, c17
dp3 r10.x, r0, r2			// N.L
lit r11, r10				// .y = max(N.L, 0)  .z = max(N.H, 0)**spow

mov oD1, c17
mov oT0, r11.y


